@@ -2,9 +2,9 @@
 # =============================================================================
 # Copyright (c) 2009 Tom Kralidis
 #
-# Authors : Tom Kralidis <tomkralidis@hotmail.com>
+# Authors : Tom Kralidis <tomkralidis@gmail.com>
 #
-# Contact email: tomkralidis@hotmail.com
+# Contact email: tomkralidis@gmail.com
 # =============================================================================
 
 """ CSW request and response processor """
@@ -323,15 +323,15 @@ class CatalogueServiceWeb:
         
             etree.SubElement(node1, util.nspath_eval('csw:ElementSetName', namespaces)).text = esn
 
-            if len(constraints) > 0: 
+            if any([len(constraints) > 0, cql is not None]): 
                 node2 = etree.SubElement(node1, util.nspath_eval('csw:Constraint', namespaces))
                 node2.set('version', '1.1.0')
                 flt = fes.FilterRequest()
-                node2.append(flt.setConstraintList(constraints))
-
-            # Now add a CQL filter if passed in
-            if cql is not None:
-                etree.SubElement(node2, util.nspath_eval('csw:CqlText', namespaces)).text = cql
+                if len(constraints) > 0:
+                    node2.append(flt.setConstraintList(constraints))
+                # Now add a CQL filter if passed in
+                elif cql is not None:
+                    etree.SubElement(node2, util.nspath_eval('csw:CqlText', namespaces)).text = cql
                 
             if sortby is not None and isinstance(sortby, fes.SortBy):
                 node1.append(sortby)
@@ -349,8 +349,15 @@ class CatalogueServiceWeb:
             val = self._exml.find(util.nspath_eval('csw:SearchResults', namespaces)).attrib.get('numberOfRecordsReturned')
             self.results['returned'] = int(util.testXMLValue(val, True))
             val = self._exml.find(util.nspath_eval('csw:SearchResults', namespaces)).attrib.get('nextRecord')
-            self.results['nextrecord'] = int(util.testXMLValue(val, True))
-    
+            if val is not None:
+                 self.results['nextrecord'] = int(util.testXMLValue(val, True))
+            else:
+                warnings.warn("""CSW Server did not supply a nextRecord value (it is optional), so the client
+                should page through the results in another way.""")
+                # For more info, see:
+                # https://github.com/geopython/OWSLib/issues/100
+                self.results['nextrecord'] = None
+
             # process list of matching records
             self.records = {}
 
